@@ -280,9 +280,12 @@ class Field(MetricsLayerBase, SQLReplacement):
                     "for the view. You can do this by adding the tag 'primary_key: true' to the "
                     "necessary dimension"
                 )
-            filters_to_apply = definition.get("filters", [])
+            # You cannot apply a filter to a field that is the same name as the field itself (this doesn't make sense)
+            filters_to_apply = [f for f in definition.get("filters", []) if f.get("field") != self.name]
 
+            else_0 = False
             if non_additive_dimension := self.non_additive_dimension:
+                else_0 = True
                 if isinstance(self.non_additive_dimension, dict):
                     filters_to_apply += [
                         {
@@ -301,7 +304,9 @@ class Field(MetricsLayerBase, SQLReplacement):
                                 "value": LiteralValue(f"{self.non_additive_cte_alias()}.{window_alias}"),
                             }
                         ]
-            definition["sql"] = Filter.translate_looker_filters_to_sql(definition["sql"], filters_to_apply)
+            definition["sql"] = Filter.translate_looker_filters_to_sql(
+                definition["sql"], filters_to_apply, else_0=else_0
+            )
 
         if (
             "sql" in definition
@@ -1293,6 +1298,12 @@ class Field(MetricsLayerBase, SQLReplacement):
                     if "field" in f and isinstance(f["field"], str) and "." not in f["field"]:
                         f["field"] = f"{self.view.name}.{f['field']}"
 
+                    if "field" in f and f["field"] == f"{self.view.name}.{self.name}":
+                        errors.append(
+                            f"Field {self.name} in view {self.view.name} has a filter that references itself."
+                            " This is invalid, and the filter will not be applied."
+                        )
+
                     errors.extend(
                         self.collect_field_filter_errors(
                             f, self.view.project, f"Field {self.name} filter", "view", self.view.name
