@@ -1,10 +1,11 @@
 from collections import defaultdict
+
 from pypika import AliasedQuery, Criterion
 
-from metrics_layer.core.sql.query_base import MetricsLayerQueryBase
-from metrics_layer.core.model.filter import LiteralValueCriterion
-from metrics_layer.core.sql.query_dialect import query_lookup, if_null_lookup
 from metrics_layer.core.model.definitions import Definitions
+from metrics_layer.core.model.filter import LiteralValueCriterion
+from metrics_layer.core.sql.query_base import MetricsLayerQueryBase
+from metrics_layer.core.sql.query_dialect import if_null_lookup, query_lookup
 
 
 class MetricsLayerMergedResultsQuery(MetricsLayerQueryBase):
@@ -40,13 +41,18 @@ class MetricsLayerMergedResultsQuery(MetricsLayerQueryBase):
             if i == 0:
                 base_cte_query = base_cte_query.from_(AliasedQuery(join_hash))
             else:
-                criteria = self._build_join_criteria(self.join_hashes[0], join_hash)
-                base_cte_query = base_cte_query.outer_join(AliasedQuery(join_hash)).on(criteria)
+                no_dimensions = all(len(v) == 0 for v in self.query_dimensions.values())
+                # We have to do this because Redshift doesn't support a full outer join
+                # of two CTE's without dimensions using 1=1
+                if self.query_type == Definitions.redshift and no_dimensions:
+                    base_cte_query = base_cte_query.join(AliasedQuery(join_hash)).cross()
+                else:
+                    criteria = self._build_join_criteria(self.join_hashes[0], join_hash, no_dimensions)
+                    base_cte_query = base_cte_query.outer_join(AliasedQuery(join_hash)).on(criteria)
 
         return base_cte_query
 
-    def _build_join_criteria(self, first_query_alias, second_query_alias):
-        no_dimensions = all(len(v) == 0 for v in self.query_dimensions.values())
+    def _build_join_criteria(self, first_query_alias, second_query_alias, no_dimensions: bool):
         # No dimensions to join on, the query results must be just one number each
         if no_dimensions:
             return LiteralValueCriterion("1=1")
@@ -97,6 +103,7 @@ class MetricsLayerMergedResultsQuery(MetricsLayerQueryBase):
                     )
 
         dimension_sql = {}
+        all_dimension_ids = [field.id() for fields in self.query_dimensions.values() for field in fields]
         for join_hash, field_set in sorted(self.query_dimensions.items()):
             for field in field_set:
                 alias = field.alias(with_view=True)
@@ -107,8 +114,11 @@ class MetricsLayerMergedResultsQuery(MetricsLayerQueryBase):
                     dimension_sql[alias] = f"{if_null_func}({dimension_sql[alias]}, {join_hash}.{alias})"
 
                 if field.id() in mapping_lookup:
+                    present_fields = [
+                        f for f in mapping_lookup[field.id()] if f["field"] in all_dimension_ids
+                    ]
                     if_null_func = if_null_lookup[self.query_type]
-                    nested_sql = self.nested_if_null(mapping_lookup[field.id()], if_null_func)
+                    nested_sql = self.nested_if_null(present_fields, if_null_func)
                     dimension_sql[alias] = f"{if_null_func}({join_hash}.{alias}, {nested_sql})"
 
         for alias, sql in dimension_sql.items():
